"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.redirect = exports.notFound = void 0;
const next_runtime_1 = require("next-runtime");
const errors_1 = require("./errors");
class RedirectResponse {
    constructor(destination, statusCode, headers) {
        this.destination = destination;
        this.statusCode = statusCode;
        this.headers = headers;
    }
}
const handle = (handler) => {
    return (ctx) => __awaiter(void 0, void 0, void 0, function* () {
        var _a;
        ctx.req.body = (_a = ctx.req.body) !== null && _a !== void 0 ? _a : {};
        try {
            const response = yield handler(ctx);
            return (0, next_runtime_1.json)(JSON.parse(JSON.stringify(response)));
        }
        catch (err) {
            if (err instanceof RedirectResponse) {
                return (0, next_runtime_1.redirect)(err.destination, { status: err.statusCode, headers: err.headers });
            }
            if (!(err instanceof errors_1.APIError)) {
                err = new errors_1.InternalServerError();
            }
            if (err instanceof errors_1.NotFoundError) {
                return (0, next_runtime_1.notFound)({ status: err.statusCode, headers: err.headers });
            }
            return (0, next_runtime_1.json)(err.toJSON(), err.statusCode);
        }
    });
};
const notFound = (headers = {}) => {
    const response = new errors_1.NotFoundError();
    response.headers = headers;
    throw response;
};
exports.notFound = notFound;
const redirect = (destination, status) => {
    throw new RedirectResponse(destination, typeof status === 'number' ? status : typeof status === 'object' ? status.statusCode : 302, typeof status === 'object' ? status.headers : {});
};
exports.redirect = redirect;
exports.default = (handlers) => {
    const runtimeHanders = {};
    if (handlers.loader) {
        runtimeHanders.get = handle(handlers.loader);
    }
    if (handlers.action) {
        runtimeHanders.post = handle(handlers.action);
    }
    if (handlers.action && !handlers.loader) {
        runtimeHanders.get = () => ({ props: {} });
    }
    return (0, next_runtime_1.handle)(runtimeHanders);
};
//# sourceMappingURL=runtime.js.map